/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Properties;
import java.util.Set;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.MissingServerPropertyException;
import org.ow2.petals.microkernel.api.configuration.exception.NoServerPropertiesProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoTopologyProvidedException;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.SubdomainMode;
import org.ow2.petals.topology.generated.Topology;

/**
 * Unit tests of {@link ConfigurationServiceImpl#getSubDomainConfiguration()},
 * {@link ConfigurationServiceImpl#getSubDomainConfiguration(String)} and
 * {@link ConfigurationServiceImpl#getSubDomainsConfiguration()}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationServiceImpl_GetSubdomainConfigurationTest extends AbstractConfigurationServiceImplTest {

    private final static String SUBDOMAIN_NAME_1 = "subdomain-1";

    private final static SubdomainMode SUBDOMAIN_MODE_1 = SubdomainMode.DYNAMIC;

    private final static String SUBDOMAIN_DESCRIPTION_1 = "A sample sub-domain 1 configuration";

    @Rule
    public final LocalServerPropertiesBuilder localServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    /**
     * <p>
     * Check {@link ConfigurationServiceImpl#getSubDomainConfiguration()}
     * </p>
     * <p>
     * Expected result: the {@link SubDomainConfiguration} returned is correctly initialized
     * </p>
     */
    @Test
    public void testGetSubDomainConfiguration() throws InvocationTargetException, NoTopologyProvidedException,
            NoServerPropertiesProvidedException, MissingServerPropertyException, ConfigurationException, IOException {

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        try {
            cfgSvc.loadConfiguration(serverProps, this.topology);

            final SubDomainConfiguration subdomainConfig = cfgSvc.getSubDomainConfiguration();
            assertNotNull("No subdomain configuration found", subdomainConfig);
            assertEquals("Unexpected sub-domain name", ConfigurationTestUtils.SUBDOMAIN_NAME_0,
                    subdomainConfig.getName());
            assertEquals("Unexpected sub-domain description", ConfigurationTestUtils.SUBDOMAIN_DESCRIPTION_0,
                    subdomainConfig.getDescription());
            assertEquals("Unexpected sub-domain mode", SubDomainConfiguration.SubdomainMode.STATIC,
                    subdomainConfig.getMode());
            assertFalse("Unexpected network time synchronized", subdomainConfig.isNetworkTimeSynchronized());
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check {@link ConfigurationServiceImpl#getSubDomainConfiguration(String)} against a topology with two sub-domains
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>the current sub-domain configuration is return, correctly initialize, when a '<code>null</code>' sub-domain
     * name is given as parameter,</li>
     * <li>the current sub-domain configuration is return, correctly initialize, when invoked with its sub-domain name,</li>
     * <li>the other sub-domain configuration is return, correctly initialize, when invoked with its sub-domain name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testGetSubDomainConfigurationWithNullSubdomainName() throws InvocationTargetException,
            NoTopologyProvidedException, NoServerPropertiesProvidedException, MissingServerPropertyException,
            ConfigurationException, IOException {

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        final Topology topology = ConfigurationTestUtils.createBasicSampleTopology();

        final Subdomain subdomain = new Subdomain();
        topology.getDomain().getSubDomain().add(subdomain);
        subdomain.setName(SUBDOMAIN_NAME_1);
        subdomain.setMode(SUBDOMAIN_MODE_1);
        subdomain.setDescription(SUBDOMAIN_DESCRIPTION_1);
        subdomain.setNetworkTimeSynchronized(Boolean.TRUE);

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        try {
            cfgSvc.loadConfiguration(serverProps, topology);

            final Set<SubDomainConfiguration> subdomainCfgs = cfgSvc.getSubDomainsConfiguration();
            assertNotNull("sub-doain configuration list is null", subdomainCfgs);
            assertEquals("Unexpected sub-domain number", 2, subdomainCfgs.size());

            // Check invocation using a sub-domain name 'null'
            final SubDomainConfiguration subdomainConfig_Null = cfgSvc.getSubDomainConfiguration(null);
            assertNotNull("No subdomain configuration found", subdomainConfig_Null);
            assertEquals("Unexpected sub-domain name", ConfigurationTestUtils.SUBDOMAIN_NAME_0,
                    subdomainConfig_Null.getName());
            assertEquals("Unexpected sub-domain description", ConfigurationTestUtils.SUBDOMAIN_DESCRIPTION_0,
                    subdomainConfig_Null.getDescription());
            assertEquals("Unexpected sub-domain mode", SubDomainConfiguration.SubdomainMode.STATIC,
                    subdomainConfig_Null.getMode());
            assertFalse("Unexpected network time synchronized", subdomainConfig_Null.isNetworkTimeSynchronized());

            // Check invocation using the current sub-domain name
            final SubDomainConfiguration subdomainConfig_Current = cfgSvc
                    .getSubDomainConfiguration(ConfigurationTestUtils.SUBDOMAIN_NAME_0);
            assertNotNull("No subdomain configuration found", subdomainConfig_Current);
            assertEquals("Unexpected sub-domain name", ConfigurationTestUtils.SUBDOMAIN_NAME_0,
                    subdomainConfig_Current.getName());
            assertEquals("Unexpected sub-domain description", ConfigurationTestUtils.SUBDOMAIN_DESCRIPTION_0,
                    subdomainConfig_Current.getDescription());
            assertEquals("Unexpected sub-domain mode", SubDomainConfiguration.SubdomainMode.STATIC,
                    subdomainConfig_Current.getMode());
            assertFalse("Unexpected network time synchronized", subdomainConfig_Current.isNetworkTimeSynchronized());

            // Check invocation using the other sub-domain name
            final SubDomainConfiguration subdomainConfig_Other = cfgSvc.getSubDomainConfiguration(SUBDOMAIN_NAME_1);
            assertNotNull("No subdomain configuration found", subdomainConfig_Other);
            assertEquals("Unexpected sub-domain name", SUBDOMAIN_NAME_1, subdomainConfig_Other.getName());
            assertEquals("Unexpected sub-domain description", SUBDOMAIN_DESCRIPTION_1,
                    subdomainConfig_Other.getDescription());
            assertEquals("Unexpected sub-domain mode", SubDomainConfiguration.SubdomainMode.DYNAMIC,
                    subdomainConfig_Other.getMode());
            assertTrue("Unexpected network time synchronized", subdomainConfig_Other.isNetworkTimeSynchronized());
        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }
}
