/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration.SubdomainMode;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoSubdomainNameProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainAlreadyExistException;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.Topology;

/**
 * Unit tests of {@link ConfigurationServiceImpl#reconfigureSubdomain(SubDomainConfiguration)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationServiceImpl_ReconfigureSubdomainTest {

    private final static String SUBDOMAIN_NAME_1 = "subdomain-1";

    private final static SubdomainMode SUBDOMAIN_MODE_1 = SubdomainMode.STATIC;

    private final static String SUBDOMAIN_DESCRIPTION_1 = "A sample sub-domain 1 configuration";

    private final static boolean SUBDOMAIN_NETWORK_1 = true;

    private final static List<Object> SUBDOMAIN_EXTRA_PARAM_1 = new ArrayList<Object>();

    @Rule
    public final LocalServerPropertiesBuilder localServerPropertiesBuilder = new LocalServerPropertiesBuilder();

    /**
     * <p>
     * Check the reconfiguration of a sub-domain against:
     * <ul>
     * <li>no sub-domain configuration is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: A error occurs because no sub-domain configuration is provided.
     * </p>
     */
    @Test(expected = NoSubdomainConfigurationProvidedException.class)
    public void testNoSubdomainProvided() throws ConfigurationException, InvocationTargetException, IOException {

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        try {
            final Topology topology = ConfigurationTestUtils.createBasicSampleTopology();

            cfgSvc.loadConfiguration(serverProps, topology);

            cfgSvc.reconfigureSubdomain(null);

        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check the reconfiguration of a sub-domain against:
     * <ul>
     * <li>the new sub-domain name is not provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: A error occurs because the sub-domain name is not provided.
     * </p>
     */
    @Test(expected = NoSubdomainNameProvidedException.class)
    public void testSubdomainNameNotProvided() throws ConfigurationException, InvocationTargetException, IOException {

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        try {
            final Topology topology = ConfigurationTestUtils.createBasicSampleTopology();

            cfgSvc.loadConfiguration(serverProps, topology);

            final SubDomainConfiguration newSubdomainConfiguration = new SubDomainConfiguration();

            cfgSvc.reconfigureSubdomain(newSubdomainConfiguration);

        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check the reconfiguration of a sub-domain against:
     * <ul>
     * <li>the new sub-domain name already exists</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: A error occurs because the sub-domain name already exists.
     * </p>
     */
    @Test(expected = SubdomainAlreadyExistException.class)
    public void testSubdomainNameAlreadyExists() throws ConfigurationException, InvocationTargetException, IOException {

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        try {
            final Topology topology = ConfigurationTestUtils.createBasicSampleTopology();

            final Subdomain subdomain = new Subdomain();
            topology.getDomain().getSubDomain().add(subdomain);
            subdomain.setName(SUBDOMAIN_NAME_1);
            subdomain.setMode(org.ow2.petals.topology.generated.SubdomainMode.DYNAMIC);
            subdomain.setDescription(SUBDOMAIN_DESCRIPTION_1);

            cfgSvc.loadConfiguration(serverProps, topology);

            final SubDomainConfiguration newSubdomainConfiguration = new SubDomainConfiguration();
            newSubdomainConfiguration.setName(SUBDOMAIN_NAME_1);

            cfgSvc.reconfigureSubdomain(newSubdomainConfiguration);

        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }

    /**
     * <p>
     * Check the reconfiguration of a sub-domain against:
     * <ul>
     * <li>all is ok</li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>No error occurs.</li>
     * <li>the new sub-domain configuration exists in the sub-domain list</li>
     * <li>the initial sub-domain configuration no more exist in the sub-domain list</li>
     * </ul>
     * </p>
     */
    @Test
    public void testAllIsOk() throws ConfigurationException, InvocationTargetException, IOException {

        final ConfigurationService cfgSvc = new ConfigurationServiceImpl();
        ConfigurationTestUtils.startConfigurationService(cfgSvc);

        final Properties serverProps = this.localServerPropertiesBuilder
                .createMinimalServerProperties(ConfigurationTestUtils.CONTAINER_NAME_0);

        try {
            final Topology topology = ConfigurationTestUtils.createBasicSampleTopology();

            cfgSvc.loadConfiguration(serverProps, topology);

            final SubDomainConfiguration newSubdomainConfiguration = new SubDomainConfiguration();
            newSubdomainConfiguration.setName(SUBDOMAIN_NAME_1);
            newSubdomainConfiguration.setMode(SUBDOMAIN_MODE_1);
            newSubdomainConfiguration.setDescription(SUBDOMAIN_DESCRIPTION_1);
            newSubdomainConfiguration.setNetworkTimeSynchronized(SUBDOMAIN_NETWORK_1);
            newSubdomainConfiguration.setExtraParameters(SUBDOMAIN_EXTRA_PARAM_1);

            cfgSvc.reconfigureSubdomain(newSubdomainConfiguration);
            
            final SubDomainConfiguration reconfiguredSubdomain = cfgSvc.getSubDomainConfiguration();
            assertEquals("Unreconfigured sub-domain name", SUBDOMAIN_NAME_1, reconfiguredSubdomain.getName());
            assertEquals("Unreconfigured sub-domain description", SUBDOMAIN_DESCRIPTION_1, reconfiguredSubdomain.getDescription());
            assertEquals("Unreconfigured sub-domain mode", SUBDOMAIN_MODE_1, reconfiguredSubdomain.getMode());
            assertEquals("Unreconfigured sub-domain network time sync", SUBDOMAIN_NETWORK_1, reconfiguredSubdomain.isNetworkTimeSynchronized());
            assertEquals("Unreconfigured sub-domain extra param", SUBDOMAIN_EXTRA_PARAM_1, reconfiguredSubdomain.getExtraParameters());
            
            final SubDomainConfiguration retrievedReconfiguredSubdomain = cfgSvc.getSubDomainConfiguration(SUBDOMAIN_NAME_1);
            assertNotNull("reconfigured sub-domain not found", retrievedReconfiguredSubdomain);
            assertEquals("Unreconfigured sub-domain name", SUBDOMAIN_NAME_1, retrievedReconfiguredSubdomain.getName());
            assertEquals("Unreconfigured sub-domain description", SUBDOMAIN_DESCRIPTION_1, retrievedReconfiguredSubdomain.getDescription());
            assertEquals("Unreconfigured sub-domain mode", SUBDOMAIN_MODE_1, retrievedReconfiguredSubdomain.getMode());
            assertEquals("Unreconfigured sub-domain network time sync", SUBDOMAIN_NETWORK_1, retrievedReconfiguredSubdomain.isNetworkTimeSynchronized());
            assertEquals("Unreconfigured sub-domain extra param", SUBDOMAIN_EXTRA_PARAM_1, retrievedReconfiguredSubdomain.getExtraParameters());
            
            assertNull("initial sub-domain retrieved", cfgSvc.getSubDomainConfiguration(ConfigurationTestUtils.SUBDOMAIN_NAME_0));

        } finally {
            ConfigurationTestUtils.stopConfigurationService(cfgSvc);
        }
    }
}
